==> ikchatbot-1.0.0.tar.gz: lib/src/ikchatbot.dart <==
import 'dart:async';
import 'package:flutter/material.dart';
import 'chat_message.dart';
import 'ike_chatbot_config.dart';

class IkChatBot extends StatefulWidget {
  final IkChatBotConfig config;

  const IkChatBot({Key? key, required this.config}) : super(key: key);

  @override
  State<IkChatBot> createState() => _IkChatBotState();
}

class _IkChatBotState extends State<IkChatBot> {
  final List<ChatMessage> _messages = [];
  final TextEditingController _textController = TextEditingController();

  Timer? _inactivityTimer;
  Timer? _closeTimer;
  bool _isWaitingForUserResponse = true;
  bool _showTextField = true;

  @override
  void initState() {
    super.initState();
    _messages.add(ChatMessage(
      botColor: widget.config.botChatColor,
      botIcon: widget.config.botIcon,
      userColor: widget.config.userChatColor,
      userIcon: widget.config.userIcon,
      text: widget.config.initialGreeting, // Use the initial greeting from the config
      isUser: false,
      messageTime: DateTime.now(),
    ));

    _startInactivityTimer();
  }

  void _handleSubmitted(String text) {
    _inactivityTimer?.cancel();
    _closeTimer?.cancel();

    if (_isWaitingForUserResponse) {
      _handleUserResponse(text);
      return;
    }

    if (text.trim().isEmpty) return;

    setState(() {
      _messages.add(ChatMessage(
        botColor: widget.config.botChatColor,
        botIcon: widget.config.botIcon,
        userColor: widget.config.userChatColor,
        userIcon: widget.config.userIcon,
        text: text,
        isUser: true,
        messageTime: DateTime.now(),
      ));
    });

    _textController.clear();

    Future.delayed( Duration(minutes: widget.config.closingTime), () {
      _handleUserResponse(text);
    });
  }

  int _findMatchingKeyword(String response) {
    for (int i = 0; i < widget.config.keywords.length; i++) {
      if (response.contains(widget.config.keywords[i])) {
        return i;
      }
    }
    return -1;
  }

  void _handleUserResponse(String response) {
    final lowerCaseResponse = response.trim().toLowerCase();

    String reply;
    String userSelectedOption;

    final int index = _findMatchingKeyword(lowerCaseResponse);

    if (index != -1) {
      userSelectedOption = response;
      reply = widget.config.responses[index];
    } else {
      userSelectedOption = "";
      reply = widget.config.defaultResponse;
    }

    if (userSelectedOption.isNotEmpty) {
      setState(() {
        _messages.add(ChatMessage(
          botColor: widget.config.botChatColor,
          botIcon: widget.config.botIcon,
          userColor: widget.config.userChatColor,
          userIcon: widget.config.userIcon,
          text: userSelectedOption,
          isUser: true,
          messageTime: DateTime.now(),
        ));
      });
    }

    setState(() {
      _isWaitingForUserResponse = false;
    });

    // Delay for 1 second before adding the robot's reply
    Future.delayed(Duration(seconds: widget.config.delayResponse), () {
      setState(() {
        _messages.add(ChatMessage(
          botColor: widget.config.botChatColor,
          botIcon: widget.config.botIcon,
          userColor: widget.config.userChatColor,
          userIcon: widget.config.userIcon,
          text: reply,
          isUser: false,
          messageTime: DateTime.now(),
        ));
        _startInactivityTimer();
      });
      _textController.clear();
    });
  }

  void _startInactivityTimer() {
    _inactivityTimer = Timer( Duration(minutes: widget.config.waitingTime), () {
      if (!_isWaitingForUserResponse) {
        setState(() {
          _messages.add(ChatMessage(
            botColor: widget.config.botChatColor,
            botIcon: widget.config.botIcon,
            userColor: widget.config.userChatColor,
            userIcon: widget.config.userIcon,
            text: widget.config.inactivityMessage, // Use inactivity message from the config
            isUser: false,
            messageTime: DateTime.now(),
          ));
          _isWaitingForUserResponse = true;
          _startInactivityTimer();
        });
      } else {
        _startCloseTimer();
      }
    });
  }

  void _startCloseTimer() {
    _closeTimer = Timer(const Duration(minutes: 5), () {
      setState(() {
        _showTextField = false;
        _messages.add(ChatMessage(
          botColor: widget.config.botChatColor,
          botIcon: widget.config.botIcon,
          userColor: widget.config.userChatColor,
          userIcon: widget.config.userIcon,
          text: widget.config.closingMessage, // Use closing message from the config
          isUser: false,
          messageTime: DateTime.now(),
        ));
      });
    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: Container(
        decoration: BoxDecoration(
          image: DecorationImage(
            image: NetworkImage(widget.config.backgroundImageUrl), // Use background image URL from the config
            fit: BoxFit.cover,
          ),
        ),
        child: Column(
          children: [
            Expanded(
              child: Align(
                alignment: Alignment.topCenter,
                child: ListView.builder(
                  reverse: true,
                  itemBuilder: (_, int index) =>
                  _messages.reversed.toList()[index],
                  itemCount: _messages.length,
                ),
              ),
            ),
            const Divider(height: 1.0),
            _buildTextComposer(),
          ],
        ),
      ),
    );
  }

  Widget _buildTextComposer() {
    if (!_showTextField) {
      return const SizedBox.shrink();
    }

    return Padding(
      padding: const EdgeInsets.all(8.0),
      child: Container(
        decoration: BoxDecoration(
          color: widget.config.backgroundColor, // Use background color from the config
          borderRadius: BorderRadius.circular(5.0),
        ),
        padding: const EdgeInsets.symmetric(horizontal: 8.0),
        child: Row(
          children: [
            Flexible(
              child: TextField(
                controller: _textController,
                onChanged: (text) {
                  if (!_isWaitingForUserResponse) {
                    _isWaitingForUserResponse = true;
                  }
                },
                decoration: InputDecoration.collapsed(
                  hintText: widget.config.inputHint, // Use input hint from the config
                ),
              ),
            ),
            IconButton(
              icon: const Icon(Icons.send),
              onPressed: () {
                _handleSubmitted(_textController.text);
              },
            ),
          ],
        ),
      ),
    );
  }
}

==> ikchatbot-1.0.2.tar.gz: lib/src/ikchatbot.dart <==
import 'dart:async';
import 'package:flutter/material.dart';
import 'package:ikchatbot/src/thank_you.dart';
import '../ikchatbot.dart';


class IkChatBot extends StatefulWidget {
  final IkChatBotConfig config;

  const IkChatBot({Key? key, required this.config}) : super(key: key);

  @override
  State<IkChatBot> createState() => _IkChatBotState();
}

class _IkChatBotState extends State<IkChatBot> {
  final List<ChatMessage> _messages = [];
  final TextEditingController _textController = TextEditingController();

  final ScrollController _scrollController = ScrollController();

  Timer? _inactivityTimer;
  Timer? _closeTimer;
  bool _isWaitingForUserResponse = true;
  bool _conversationOver = false;

  @override
  void initState() {
    super.initState();
    _addBotMessage(widget.config.initialGreeting);
    _startInactivityTimer();
  }

  void _addBotMessage(String text) {
    final botMessage = ChatMessage(
      botColor: widget.config.botChatColor,
      botIcon: widget.config.botIcon,
      userColor: widget.config.userChatColor,
      userIcon: widget.config.userIcon,
      text: text,
      isUser: false,
      messageTime: DateTime.now(),
    );
    setState(() {
      _messages.add(botMessage);
    });
  }

  void _handleSubmitted(String text) {
    _inactivityTimer?.cancel();
    _closeTimer?.cancel();

    if (_isWaitingForUserResponse) {
      _handleUserResponse(text);
      return;
    }

    if (text.trim().isEmpty) return;

    setState(() {
      _messages.add(ChatMessage(
        botColor: widget.config.botChatColor,
        botIcon: widget.config.botIcon,
        userColor: widget.config.userChatColor,
        userIcon: widget.config.userIcon,
        text: text,
        isUser: true,
        messageTime: DateTime.now(),
      ));
    });

    _textController.clear();

    // Scroll to the end of the list after adding a new message
    _scrollController.animateTo(
      _scrollController.position.maxScrollExtent,
      duration: Duration(milliseconds: widget.config.delayBot),
      curve: Curves.easeOut,
    );

    Future.delayed(Duration(minutes: widget.config.closingTime), () {
      _handleUserResponse(text);
    });
  }

  void _handleUserResponse(String response) {
    final lowerCaseResponse = response.trim().toLowerCase();

    String reply;
    String userSelectedOption;

    final int index = _findMatchingKeyword(lowerCaseResponse);

    if (index != -1) {
      userSelectedOption = response;
      reply = widget.config.responses[index];
    } else {
      userSelectedOption = response;
      reply = widget.config.defaultResponse;
    }

    setState(() {
      _messages.add(ChatMessage(
        botColor: widget.config.botChatColor,
        botIcon: widget.config.botIcon,
        userColor: widget.config.userChatColor,
        userIcon: widget.config.userIcon,
        text: userSelectedOption,
        isUser: true,
        messageTime: DateTime.now(),
      ));

      _textController.clear();
    });

    if (reply.isNotEmpty) {
      _isWaitingForUserResponse = false;
      _startTypingAnimation(reply);
    }
  }

  int _findMatchingKeyword(String response) {
    for (int i = 0; i < widget.config.keywords.length; i++) {
      if (response.contains(widget.config.keywords[i])) {
        return i;
      }
    }
    return -1;
  }

  void _startTypingAnimation(String reply) {
    final typingMessage = ChatMessage(
      botColor: widget.config.botChatColor,
      botIcon: widget.config.botIcon,
      userColor: widget.config.userChatColor,
      userIcon: widget.config.userIcon,
      text: widget.config.waitingText,
      isUser: false,
      messageTime: DateTime.now(),
    );

    setState(() {
      _messages.add(typingMessage);
    });

    Future.delayed(Duration(seconds: widget.config.delayResponse), () {
      // Simulating a delay for the typing animation
      _messages.remove(typingMessage); // Remove the "Typing..." message
      _addBotMessage(reply); // Add the actual bot response
      _startInactivityTimer(); // Start the inactivity timer

      if (reply == widget.config.closingMessage) {
        setState(() {
          _conversationOver = true;
        });
      }
    });
  }

  void _startInactivityTimer() {
    _inactivityTimer = Timer(Duration(minutes: widget.config.waitingTime), () {
      if (!_isWaitingForUserResponse) {
        _addBotMessage(widget.config.inactivityMessage);
        _isWaitingForUserResponse = true;
        _startInactivityTimer();
      } else {
        _startCloseTimer();
      }
    });
  }

  void _startCloseTimer() {
    _closeTimer = Timer(Duration(minutes: widget.config.closingTime), () {
      setState(() {
        _conversationOver = true;  // Mark conversation as over
        _addBotMessage(widget.config.closingMessage);  // Add closing message
      });
    });
  }


  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: _conversationOver
          ?  ThankYouWidget(
ratingBackgroundColor: widget.config.ratingBackgroundColor,
        ratingIconColor: widget.config.ratingIconColor,
        ratingIconYes: widget.config.ratingIconYes,
        ratingTitle: widget.config.ratingTitle,
        ratingText: widget.config.ratingText,
        smtpPort: widget.config.smtpPort,
        smtpServer: widget.config.smtpServer,
        smtpUsername: widget.config.smtpUsername,
        smtpPassword: widget.config.smtpPassword,
        senderName: widget.config.senderName,
        isSecure: widget.config.isSecure,
        recipient: widget.config.recipient,
        subject: widget.config.subject,
        body: widget.config.body,
        ratingIconNo: widget.config.ratingIconNo,
        thankYouText: widget.config.thankyouText,


      )  // Display ThankYouWidget if conversation is over
          : Container(
        decoration: BoxDecoration(
          image: DecorationImage(
            image: NetworkImage(widget.config.backgroundImageUrl),
            fit: BoxFit.cover,
          ),
        ),
        child: Column(
          children: [
            Expanded(
              child: Align(
                alignment: Alignment.bottomCenter,
                child: ListView.builder(
                  controller: _scrollController,
                  reverse: true,
                  itemBuilder: (_, int index) =>
                  _messages.reversed.toList()[index],
                  itemCount: _messages.length,
                ),
              ),
            ),
            const Divider(height: 1.0),
            _buildTextComposer(),
          ],
        ),
      ),
    );
  }

  Widget _buildTextComposer() {
    return Padding(
      padding: const EdgeInsets.all(8.0),
      child: Container(
        decoration: BoxDecoration(
          color: widget.config.backgroundColor,
          borderRadius: BorderRadius.circular(5.0),
        ),
        padding: const EdgeInsets.symmetric(horizontal: 8.0),
        child: Row(
          children: [
            Flexible(
              child: TextField(
                controller: _textController,
                onChanged: (text) {
                  if (!_isWaitingForUserResponse) {
                    _isWaitingForUserResponse = true;
                  }
                },
                decoration: InputDecoration.collapsed(
                  hintText: widget.config.inputHint,
                ),
              ),
            ),
            IconButton(
              icon: const Icon(Icons.send),
              onPressed: () {
                _handleSubmitted(_textController.text);
              },
            ),
          ],
        ),
      ),
    );
  }

  @override
  void dispose() {
    _inactivityTimer?.cancel();
    _closeTimer?.cancel();
    super.dispose();
  }
}
